//----------------------------------------------------------------------------//
// Programa.....: Mando_IR_canales.c (remap para usar usb-bootloader 48Mhz    //
//                usando el linker: rm18f4550 - HID Bootload.lkr              //
// Plataforma hw: Placa pluma 18F4550(Xtal: 20Mhz) + placa perifericos        //
// Fecha........: enero-2013                                                  //
// Programador..: Domingo Llorente                                            //
// Descripcin..: Programa que enciende/apaga la barra de leds de la placa    //
// cuando se pulsan las teclas de volumen (+/-) y muestra el nombre del canal //
// en el lcd cuando se pulsan las teclas de canal+/canal- del mando Sony      //
// -------------------------------------------------------------------------- //
//                      T           T       T       T           T             //
// ----                ---         ---     ---     ---         ---         -  //
//     !              !   !       !   !   !   !   !   !       !   !       !   //
//     !___2500us_____!   !_T_._T_!   !.T.!   !.T.!   !_T_._T_!   !_T_._T_!   //
//                                                                            //
//                    |--(dato 1)-| dato 0| dato 0|--(dato 1)-|--(dato 1)-|   //
//                                                                            //
// El cdigo SONY manda 12 bits, considerando un "1" cuando est en alto 1T y //
// en bajo 2T y un "0" cuando est en alto 1T y en bajo 1T.                   //
// Siento el valor de T=600us                                                 //
////////////////////////////////////////////////////////////////////////////////
#include <p18F4550.h>
#include <delays.h>               // funciones de retardo
#include <xlcd_pluma.h>           // funciones de control del lcd
#include "fuses_18f4550_48Mhz.c"  // fuses de configuracin del pic

//--- REMAPEO DE LOS VECTORES DE INTERRUPCIN PARA NO BORRAR EL BOOTLOADER ---/
// Este bloque es obligatorio si se utiliza el Botloader USB para cargar el   /
// programa en el pic, de lo contrario se puede eliminar.                     /
//----------------------------------------------------------------------------/
#define NEW_RESET_VECTOR_ADDRESS		0x1000
#define NEW_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
#define NEW_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018

void UserHighPriorityISRCode();
void UserLowPriorityISRCode();

extern void _startup (void);
#pragma code NEW_RESET_VECTOR = NEW_RESET_VECTOR_ADDRESS
void _reset (void){
    _asm goto _startup _endasm
}
#pragma code NEW_HIGH_INTERRUPT_VECTOR = NEW_HIGH_INTERRUPT_VECTOR_ADDRESS
void New_High_ISR (void)	{
     _asm goto UserHighPriorityISRCode _endasm
}
#pragma code NEW_LOW_INTERRUPT_VECTOR = NEW_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void){
     _asm goto UserLowPriorityISRCode _endasm
}

#pragma code HIGH_INTERRUPT_VECTOR = 0x08
void High_ISR (void)	{
     _asm goto NEW_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
}
#pragma code LOW_INTERRUPT_VECTOR = 0x18
void Low_ISR (void){
     _asm goto NEW_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
}
#pragma code
#pragma interrupt UserHighPriorityISRCode
void UserHighPriorityISRCode(){ }

#pragma interruptlow UserLowPriorityISRCode
void UserLowPriorityISRCode() { }
#pragma code
//------------ FIN DEL BLOQUE DE REMAPEO DE INTERRUPCIONES ------------------//
unsigned char numero[7];
unsigned char volumen[9]={0,1,3,7,15,31,63,127,255};
unsigned char tecla,n=0,canal=1,ok=0;
char vol=0;
unsigned char tecla_ir(void);
void mostrar_canal(unsigned char canal_tv);

void main(void)
{
ADCON1=0x0F;     // Todos entrada/salida digitales
TRISA|=0x04;     // RA2 entrada IR
TRISB=0x00;      // PORTB como salida (leds/display)
TRISC&=0xF9;     // RC1 y RC2 salidas (led bicolor)
LATB=0x00;       // Leds apagados
LATC&=0xF9;      // Led bicolor apagado

OpenTimer0(TIMER_INT_OFF&T0_SOURCE_INT&T0_16BIT&T0_PS_1_1);
lcd_init();
escribe_cadena("  Mando MG-839  ");
cambio_linea();
escribe_cadena("  Canales TV    ");
for(n=1;n<=15;n++) Delay10KTCYx(240);  // Ret. de 3seg. para 48Mhz
borrar_lcd();
escribe_cadena("(1) TVE-1");

while(1)
 {
 tecla=tecla_ir();
  switch(tecla){
  case 0: mostrar_canal(1);  break;
  case 1: mostrar_canal(2);  break;
  case 2: mostrar_canal(3);  break;
  case 3: mostrar_canal(4);  break;
  case 4: mostrar_canal(5);  break;
  case 5: mostrar_canal(6);  break;
  case 6: mostrar_canal(7);  break;
  case 7: mostrar_canal(8);  break;
  case 8: mostrar_canal(9);  break;
  case 18:
    vol++;
    if(vol<=8) LATB=volumen[vol];
    else vol=8;
    break;
  case 19:
    if(vol>0){
    vol--;
    LATB=volumen[vol];
    }
      else vol=0;
  break;
  case 16:                       // Incrementar canal
    canal++;
    if(canal<=9){
    mostrar_canal(canal);
    }
    else canal=9;
  break;
  case 17:                       // Decrementar canal
         if(canal>1){
          canal--;
          mostrar_canal(canal);
      }
      else canal=1;
  break;
  default:
      break;
  }
  Delay10KTCYx(240);      // Retardo de seguridad
  }
}

/////// FUNCIN DE CAPTURA DE LA TECLA PULSADA EN EL MANDO SONY   //////////////
unsigned char tecla_ir(void)
{
   unsigned char dato_ir=0;
   do{
     while (PORTAbits.RA2==1) {};   //  ---  Esperar flanco de bajada ---
     Delay10KTCYx(2);               //  Retardo 1666us@48Mhz para evitar rebotes
     if(PORTAbits.RA2==0) ok=1;     //  Comando vlido
     else ok=0;                     //  Comando no-vlido. Esperar nuevo flanco
    }while (ok==0);

  while (PORTAbits.RA2==0) {};      //      |____    de inicio de envo.
  for(n=0;n<7;n++)                  // Siete bits de comando
  {
   while(PORTAbits.RA2==1){};       // Esperar flanco de bajada

   INTCONbits.TMR0IF=0;             // Reset bandera de desbordamiento
   WriteTimer0(0-10800);            // Ret. de 10800x0.0833=900us
   while(INTCONbits.TMR0IF==0);     // Esperar desbordamiento

   if(PORTAbits.RA2==1) numero[n]=0;// Si nivel alto -> bit_recibido=0
   else {
         numero[n]=1;               // Si nivel bajo -> bit_recibido=1
         while(PORTAbits.RA2==0){}; // Esperar flanco de subida
        }
  }
  dato_ir=numero[6]*64+numero[5]*32+numero[4]*16+
          numero[3]*8 +numero[2]*4 +numero[1]*2 + numero[0];

  return dato_ir;
}

void mostrar_canal(unsigned char canal_tv)
{
 switch(canal_tv){
 case 1:
     gotoxy(0,0);
     escribe_cadena("(1) TVE1      ");  break;
 case 2:
     gotoxy(0,0);
     escribe_cadena("(2) TVE-2     ");  break;
 case 3:
     gotoxy(0,0);
     escribe_cadena("(3) Antena-3  ");  break;
 case 4:
     gotoxy(0,0);
     escribe_cadena("(4) Cuatro    ");  break;
 case 5:
     gotoxy(0,0);
     escribe_cadena("(5) Telecinco ");  break;
 case 6:
     gotoxy(0,0);
     escribe_cadena("(6) La Sexta  ");  break;
 case 7:
     gotoxy(0,0);
     escribe_cadena("(7) La Siete  ");  break;
 case 8:
     gotoxy(0,0);
     escribe_cadena("(8) Canal NOVA");  break;
 case 9:
     gotoxy(0,0);
     escribe_cadena("(9) Canal NE0X");  break;
 default:
     borrar_lcd();
     escribe_cadena(" ?????????? ");
 }
}

